/*
Zarafa Mail Checker
Copyright (C) 2007 Michael Erkens

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

var Zarafa = 
{
	VERSION: "1.1.1.20080624.110",
	BUILD: "110",
	ZARAFA_VERSION: "1.1.1.20080624.110",

	login: function()
	{
		this.logged_in = false;
		this.iconUpdate("disabled", Zarafa.translate("connecting"));
		this.setTooltip(Zarafa.translate("login_in"));

		if (ZarafaPrefs.get("askpassword")){

			var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
        		                .getService(Components.interfaces.nsIPromptService);
			var username = {value:ZarafaPrefs.get("username")};
			var password = {value:""};
			var check = {value:true};

			if (prompts.promptUsernameAndPassword(window, 'Zarafa', Zarafa.translate("enter_user_pass"), username, password, Zarafa.translate("store_password"), check)){
				this.savePassword(ZarafaPrefs.get("server_url"), username.value, password.value);
				ZarafaPrefs.set("username", username.value);
				ZarafaPrefs.set("askpassword", !check.value);
			}else{
				ZarafaPrefs.set("active", false);
				this.iconUpdate("disabled", "");
				return;
			}
		}

		this.iconUpdate("transfer", Zarafa.translate("authenticating"));
		var post = "username="+escape(ZarafaPrefs.get("username"))+"&password="+escape(this.getPassword())+"&external="+escape(ZarafaPrefs.get("server_cookie"));
		this.request = new XMLHttpRequest();
		this.request.open("POST", ZarafaPrefs.get("server_url")+"/index.php?external="+ZarafaPrefs.get("server_cookie"), true);
		this.request.setRequestHeader("Content-Type", "application/x-www-form-urlencoded;charset=utf-8;");
		this.request.onload = this.onLoginRequestLoad;
		this.request.onerror = this.onLoginRequestError;
		this.request.send(post);
	},

	iconUpdate: function(type, message)
	{
		document.getElementById("zarafa-icon").setAttribute("type", type);
		
		if (typeof message != "undefined"){
			var label = document.getElementById("zarafa-label");
			while(label.hasChildNodes()){
				label.removeChild(label.firstChild);
			}
			label.appendChild(document.createTextNode(message));
		}
	},
	
	setTooltip: function(message)
	{
		var tooltip = document.getElementById("zarafa-description");
		tooltip.value = message;
	},

	getCounters: function()
	{
		this.iconUpdate("transfer");
		if (this.activeCounter){
			this.activeCounter = false;
			window.clearTimeout(this.timer);
		}

		if (!this.logged_in){
			this.login();
			return;
		}
		var xml = '<zarafa>';
		xml += '<module name="notificationmodule" id="firefox_1"><action type="getcounters"></action></module>';
		xml += '</zarafa>';

		this.setTooltip(Zarafa.translate("connecting"));

		this.request = new XMLHttpRequest();
		this.request.open("POST", ZarafaPrefs.get("server_url")+"/zarafa.php?subsystem=firefox&external="+ZarafaPrefs.get("server_cookie"), true);
		this.request.setRequestHeader("Content-Type", "text/xml;charset=utf-8;");
		this.request.onload = this.onRequestLoad;
		this.request.onerror = this.onRequestError;
		this.request.send(xml);
		this.updateCount++;
	},

	updateCounters: function(xml)
	{
		this.iconUpdate("normal");
		this.setTooltip(Zarafa.translate("update_counters"));
		var valid = false;
		var count = -1;
		var unread = 0;
		try{
			var modules = xml.getElementsByTagName("module");
			for(var i in modules){
				var attributes = modules[i].attributes;
				if (attributes){
					for(var k = 0; k < attributes.length; k++){
						var item = attributes.item(k);
						if (item.nodeName == "id" && item.nodeValue == "firefox_1"){
							count = modules[i].getElementsByTagName("count")[0].firstChild.nodeValue;
							unread = modules[i].getElementsByTagName("unread")[0].firstChild.nodeValue;
						}
					}
				}
			}
		}catch(e){
			this.setTooltip(e.toString());
			return;
		}
		
		if (count==-1){
			var error = xml.getElementsByTagName("error");
			if (error.length!=0){
				var mapiError = error[0].getAttribute("mapi");
				this.iconUpdate("disable");
				this.setTooltip(Zarafa.translate("mapi_error").replace("%s", mapiError));
				switch(mapiError){
					case "0x8004011c": // MAPI_E_UNCONFIGURED used by zarafa webaccess when login failed
						ZarafaPrefs.set("askpassword", true);
						this.setTooltip(Zarafa.translate("login_failed"));
						if (confirm(Zarafa.translate("login_failed")+" "+Zarafa.translate("try_again"))){
							this.login();
						}else{
							ZarafaPrefs.set("active", false);
							this.iconUpdate("disabled", "");
						}
						return;
						break;
					default:
						alert(Zarafa.translate("mapi_error").replace("%s", mapiError));
						break;
				}
			}
		}else{
			if (typeof this.mailCount == "undefined" || typeof this.mailUnread == "undefined"){
				this.mailCount = 0;
				this.mailUnread = 0;
			}
			this.iconUpdate("normal", unread+" / "+count);//+" ("+this.updateCount+")");
			
//			alert(this.mailUnread+" < "+ unread+"\n"+this.mailCount+" < " + count);
			if (this.mailUnread<unread && this.mailCount<count && this.updateCount>1)
			{
				this.showAlert(Zarafa.translate("got_mail"),Zarafa.translate("unread_messages")+": "+unread);
			}
			this.mailCount = count;
			this.mailUnread = unread;
			this.observerService.notifyObservers(null, "zarafa:counter", unread+","+count);
		}

	},

	/* New mail indicator */
	showAlert: function(title,message)
	{
		if (ZarafaPrefs.get("show_alertbox")){

			if (!message){
				message = title;
				title = "Zarafa";
			}

			var alertsService = Components.classes["@mozilla.org/alerts-service;1"]
									.getService(Components.interfaces.nsIAlertsService);
			alertsService.showAlertNotification("chrome://zarafa/skin/img/zarafa32.png", 
								title, 
								message, 
								true, title+" - "+message, this.alertListener);
		}
	},

	alertListener: {
		observe: function(subject, topic, data) {
			switch (topic){
				case "alertclickcallback":
					Zarafa.onAlertClick(data);
					break;
				case "alertfinished":
					break;	
			}
		}
	},

	getPassword: function()
	{
		try{
			var result = "";
			var search_host = ZarafaPrefs.get("server_url");
			var search_user = ZarafaPrefs.get("username");
	
			if ("@mozilla.org/passwormanager;1" in Components.classes) {
				var passwordManager = Components.classes["@mozilla.org/passwordmanager;1"]
										.getService(Components.interfaces.nsIPasswordManager);
				var e = passwordManager.enumerator;
	
				while (e.hasMoreElements()) {
					try {
						var pass = e.getNext().QueryInterface(Components.interfaces.nsIPassword);
						if (pass.host == search_host+" (ZARAFA_FX)"){
							if (pass.user == search_user) {
								result = pass.password;
								break;
							}
						}
					} catch (ex) {
					}
				}
			}else{
				var loginManager = Components.classes["@mozilla.org/login-manager;1"]
										.getService(Components.interfaces.nsILoginManager);
				var logins = loginManager.findLogins({}, "chrome://zarafa", null, search_host);
	
				for (var i = 0; i < logins.length; i++) {
					if (logins[i].username == search_user) {
						result = logins[i].password;
						break;
					}
				}
			}
		}catch(e){
			alert("Zarafa Add-on Error:\nCORE 6765 7450 6173 7377 6F72 6400"+e);
		}
		return result;
	},

	savePassword: function(hostname, username, password)
	{
		try{
			if ("@mozilla.org/passwormanager;1" in Components.classes) {
				var passwordManager = Components.classes["@mozilla.org/passwordmanager;1"]
										.getService(Components.interfaces.nsIPasswordManager);

				passwordManager.addUser(hostname+" (ZARAFA_FX)", username, password);
			}else{
				var loginManager = Components.classes["@mozilla.org/login-manager;1"]
										.getService(Components.interfaces.nsILoginManager);
				var nsLoginInfo = new Components.Constructor("@mozilla.org/login-manager/loginInfo;1",
											Components.interfaces.nsILoginInfo, "init");
	
				var loginInfo = new nsLoginInfo("chrome://zarafa", null, hostname, username, password, "", "");
				loginManager.addLogin(loginInfo);
			}
		}catch(e){
			alert("Zarafa Add-on Error:\nCORE 7361 7665 5061 7373 776F 7264\n\n"+e);
		}
	},

	openAbout: function()
	{
		window.open('chrome://zarafa/content/about.xul','','chrome,centerscreen,modal');
	},

	openWebAccess: function()
	{
		if (ZarafaPrefs.get("auto_logon")){
			var post = "username="+escape(ZarafaPrefs.get("username"))+"&password="+escape(Zarafa.getPassword())+"&external="+escape(ZarafaPrefs.get("server_cookie"));
			var request = new XMLHttpRequest();
			request.open("POST", ZarafaPrefs.get("server_url")+"/index.php?external=ZARAFA_WEBACCESS", false);
			request.setRequestHeader("Content-Type", "application/x-www-form-urlencoded;charset=utf-8;");
			request.send(post);
		}

		switch(ZarafaPrefs.get("open_webaccess_in")){
			case "tab":
				getBrowser().selectedTab = getBrowser().addTab(ZarafaPrefs.get('server_url')+"/index.php");
				break;
			case "window":
				window.open(ZarafaPrefs.get('server_url')+"/index.php");
				break;
			case "current":
			default:
				getBrowser().loadURI(ZarafaPrefs.get('server_url')+"/index.php");
				break;
		}
	},


	/* event handlers */

	onLoad: function(e) 
	{	
		this.clientCount = 0;
		this.updateCount = 0;
		this.timer = 0;
		this.activeTimer = false;
		this.ZARAFA_VERSION = this.translate("unknown");
	
		ZarafaPrefs.prefs.setCharPref("general.useragent.extra.zarafa", "ZarafaCheck/"+Zarafa.VERSION);

		ZarafaPrefservice = Components.classes["@mozilla.org/preferences-service;1"]
				.getService(Components.interfaces.nsIPrefService)
				.getBranch("zarafa.");
		ZarafaPrefservice.QueryInterface(Components.interfaces.nsIPrefBranch2);
		ZarafaPrefservice.addObserver("", this, false);

		this.observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);

		this.observerService.addObserver(this, "zarafa:client", false);
		this.observerService.notifyObservers(null, "zarafa:client", "load");
		this.observerService.addObserver(this, "zarafa:counter", false);

		// initial login
		document.getElementById("zarafa-menu-openwebaccess").setAttribute("default", "false");
		Zarafa.changeMenuState("zarafa-menu-openwebaccess", false);
		document.getElementById("zarafa-menu-refresh").setAttribute("default", "true");

		if (ZarafaPrefs.get("active") == true){
			this.login();
		}
	},

	onUnLoad: function(e)
	{
		Zarafa.activeTimer = false;
		window.clearTimeout(Zarafa.timer);

		ZarafaPrefservice.removeObserver("", Zarafa);
		Zarafa.observerService.removeObserver(Zarafa, "zarafa:counter");
		Zarafa.observerService.removeObserver(Zarafa, "zarafa:client");
		Zarafa.observerService.notifyObservers(null, "zarafa:client", "unload");
	},

	observe: function(subject, topic, data)
	{
		if (topic == "nsPref:changed")
		{
			if (data == "login.username" || data == "server.url"){
				ZarafaPrefs.cleanPasswordManager();
				Zarafa.logged_in = false;
				ZarafaPrefs.set("askpassword", true);
			}
			
			if (data == "zarafa.active"){
				if (ZarafaPrefs.get("active")){
					Zarafa.getCounters();
				}
			}

			if (!ZarafaPrefs.get("active")){
					Zarafa.iconUpdate("disabled");
					document.getElementById("zarafa-menu-openwebaccess").setAttribute("default", "false");
					Zarafa.changeMenuState("zarafa-menu-openwebaccess", false);
					document.getElementById("zarafa-menu-refresh").setAttribute("default", "true");
			}
			return;
		};
		
		if (topic == "zarafa:client"){
			if (data=="client"){
				Zarafa.clientCount++;
			}
			if (data=="load"){
				Zarafa.clientCount = 0; // reset client count, all other clients will respond so we want to count them

				Zarafa.observerService.notifyObservers(null, "zarafa:client", "client");
			}
			if (data=="unload"){
				Zarafa.clientCount--;
			}
			return;
		}

		if (topic == "zarafa:counter"){
			var unread = parseInt(data.split(",")[0],10);
			var count = parseInt(data.split(",")[1],10);
			Zarafa.iconUpdate("newmail", unread+" / "+count);

			if (unread!=1){
				Zarafa.setTooltip(Zarafa.translate("unread_count_inbox").replace("%s",unread));
			}else{
				Zarafa.setTooltip(Zarafa.translate("unread_count_inbox_single"));
			}
			
			// restart timer
			if (ZarafaPrefs.get("check_time")>0 && ZarafaPrefs.get("active")){
				if (Zarafa.activeTimer){
					window.clearTimeout(Zarafa.timer);
				}
				Zarafa.timer = window.setTimeout("Zarafa.getCounters()", ZarafaPrefs.get("check_time"));
				Zarafa.activeTimer = true;
			}else{
				this.iconUpdate("disabled");
			}
			Zarafa.mailCount = count;
			Zarafa.mailUnread = unread;
			return;
		}
//		alert("subject: "+subject+"\ntopic: "+topic+"\ndata: "+data);
	},

	onStatusClick: function(e) 
	{
		if (document.getElementById("zarafa-menu-openwebaccess").getAttribute("default")=="true"){
			Zarafa.openWebAccess();
		}else if (document.getElementById("zarafa-menu-refresh").getAttribute("default")=="true"){
			Zarafa.getCounters();
		}
	},

	openHierarchy: function(e)
	{
		window.openDialog('chrome://zarafa/content/hierarchy.xul', 'Hierarchy', 'chrome,centerscreen,modal');
	},

	onRefreshClick: function(e)
	{
		Zarafa.getCounters();
	},

	onRequestLoad: function(e)
	{
		var xml = Zarafa.request.responseXML;
		if (xml){
			Zarafa.getZarafaVersion();

			Zarafa.updateCounters(xml);

			document.getElementById("zarafa-menu-openwebaccess").setAttribute("default", "true");
			Zarafa.changeMenuState("zarafa-menu-openwebaccess", true);
			document.getElementById("zarafa-menu-refresh").setAttribute("default", "false");
		}else{
			Zarafa.iconUpdate("error",Zarafa.translate("xml_error"));
			Zarafa.setTooltip(Zarafa.translate("xml_error"));
			document.getElementById("zarafa-menu-openwebaccess").setAttribute("default", "false");
			Zarafa.changeMenuState("zarafa-menu-openwebaccess", false);
			document.getElementById("zarafa-menu-refresh").setAttribute("default", "true");
		}

	},

	onRequestError: function(e)
	{
		document.getElementById("zarafa-menu-openwebaccess").setAttribute("default", "false");
		Zarafa.changeMenuState("zarafa-menu-openwebaccess", false);
		document.getElementById("zarafa-menu-refresh").setAttribute("default", "true");
	},

	onLoginRequestLoad: function(e)
	{
		var html = Zarafa.request.responseText;
		if (html){
			if (html.match("<title>Zarafa Webaccess</title>")==null && Zarafa.getZarafaVersion()==null){
				alert(Zarafa.translate("invalid_url")+"\n\n"+ZarafaPrefs.get("server_url"));
				ZarafaPrefs.set("active", false);
				ZarafaPrefs.set("askpassword", true);
				Zarafa.iconUpdate("disabled", "");
			}else if (html.match("<body class=\"login\">")==null){
				Zarafa.logged_in = true;
				Zarafa.getCounters();
			}else{
				var errorMsg = html.match("<p class=\"error\">(.*)</p>");
				if (errorMsg!=null){
					errorMsg = "\n\n"+Zarafa.translate("login_error_msg")+": "+errorMsg[1];	
				}else{
					errorMsg = "";
				}
				Zarafa.setTooltip(Zarafa.translate("login_failed"));
				if (confirm(Zarafa.translate("login_failed")+" "+Zarafa.translate("try_again")+errorMsg)){
					ZarafaPrefs.set("askpassword", true);
					Zarafa.login();
				}else{
					ZarafaPrefs.set("askpassword", true);
					ZarafaPrefs.set("active", false);
					Zarafa.iconUpdate("disabled", "");
				}
			}
		}else{
			Zarafa.onLoginRequestError(e);
		}
	},

	onLoginRequestError: function(e)
	{
		Zarafa.iconUpdate("error", Zarafa.translate("server_login_error"));
		Zarafa.setTooltip(Zarafa.translate("server_login_error"));

		document.getElementById("zarafa-menu-openwebaccess").setAttribute("default", "false");
		Zarafa.changeMenuState("zarafa-menu-openwebaccess", false);
		document.getElementById("zarafa-menu-refresh").setAttribute("default", "true");
	},
	
	onAlertClick: function(data)
	{
		Zarafa.openWebAccess();
	},
	
	translate: function(text)
	{
		return document.getElementById("bundle_zarafaoverlay").getString(text);
	},

	changeMenuState: function(id, state)
	{
		document.getElementById(id).setAttribute("disabled", !state?"true":"false");
	},

	getXMLValue: function(xml, tag, value)
	{
		var result = null;
		if(typeof value != "undefined"){
			result = value;
		}

		var element = xml.getElementsByTagName(tag);
		if (element && element[0]){
			if(element[0].firstChild && (typeof element[0].firstChild.nodeValue != "undefined")){
				result = element[0].firstChild.nodeValue;
			}
		}
		return result;
	},

	getZarafaVersion: function(request)
	{
		var result = null;
		
		if (typeof request == "undefined"){
			request = Zarafa.request;
		}
	
		if (request.getResponseHeader("X-Zarafa")!=null){
			result = request.getResponseHeader("X-Zarafa");
		}else{
			var txt = request.responseText;
			var version = txt.match(/<span id="version">([^\s<]*)/);
			if (version!=null){
				result = version[1];
			}
		}
		
		if (result!=null){
			Zarafa.ZARAFA_VERSION = result;
		}else{
			Zarafa.ZARAFA_VERSION = Zarafa.translate("unknown");
		}
		return result;
	},
};
