/*
Zarafa Mail Checker
Copyright (C) 2007 Michael Erkens

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

var ZarafaPrefs = {
	init: function() {
		
		this.prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

		this.keys = new Object();
		this.keys["active"] = {name:"zarafa.active", type:"BOOL", default:false};
		this.keys["username"] = {name:"zarafa.login.username", type:"CHAR", default:"user"};
		this.keys["askpassword"] = {name:"zarafa.login.askpassword", type:"BOOL", default:true, call:this.checkPassword};
		this.keys["server_url"] = {name:"zarafa.server.url", type:"CHAR", default:"http://<servername>/webaccess", call:this.checkServerURL};
		this.keys["server_cookie"] = {name:"zarafa.server.cookie", type:"CHAR", default:"ZARAFA_FIREFOX_EXT"};
		this.keys["check_time"] = {name:"zarafa.check_time", type:"INT", default:360000};
		this.keys["auto_logon"] = {name:"zarafa.login.auto_logon_webaccess", type:"BOOL", default:true};
		this.keys["open_webaccess_in"] = {name:"zarafa.webaccess.open_in", type:"CHAR", default:"tab"};
		this.keys["show_alertbox"] = {name:"zarafa.show_new_mail_alertbox", type:"BOOL", default:true};
	},

	get: function(key)
	{
		var result = null;
		if (this.keys[key]){
			if(this.prefs.prefHasUserValue(this.keys[key].name)){
				switch(this.keys[key].type){
					case "BOOL":
						result = this.prefs.getBoolPref(this.keys[key].name);
						break;
					case "INT":
						result = this.prefs.getIntPref(this.keys[key].name);
						break;
					case "CHAR":
						result = this.prefs.getCharPref(this.keys[key].name);
						break;
				}
			}else{
				result = this.keys[key].default;
			}
			
			if (this.keys[key].call){
				result = this.keys[key].call(key, "GET", result);
			}
		}else{
			alert("Zarafa Error: unknown pref '"+key+"'");
		}
		return result;
	},

	set: function(key, value)
	{
		var result = null;
		if (this.keys[key]){
			if (this.keys[key].call){
				var result = this.keys[key].call(key, "PRE-SET", value);
				if (result==null)
					return false;
			}
	
			switch(this.keys[key].type){
				case "BOOL":
					this.prefs.setBoolPref(this.keys[key].name, value);
					break;
				case "INT":
					this.prefs.setIntPref(this.keys[key].name, value);
					break;
				case "CHAR":
					this.prefs.setCharPref(this.keys[key].name, value);
					break;
			}
			
			if (this.keys[key].call){
				result = this.keys[key].call(key, "SET", result);
			}
		}else{
			alert("Zarafa Error: unknown pref '"+key+"'");
		}
	},
	
	checkServerURL: function(key, action, value)
	{
		var result = value;
	
		if (value.substr(-1)=="/"){
			result = value.substr(0, value.length-1);
		}
		return result;
	},
	
	checkPassword: function(key, action, value)
	{
		var result = value;
	
		switch(action){
			case "GET":
				break;
			case "PRE-SET":
				break;
			case "SET":
				if (value){
					ZarafaPrefs.cleanPasswordManager();
				}
				break;		
		}
	
		return result;
	},
	
	cleanPasswordManager: function()
	{
		if ("@mozilla.org/passwormanager;1" in Components.classes) {
		
			var passwordManager = Components.classes["@mozilla.org/passwordmanager;1"]
									.getService(Components.interfaces.nsIPasswordManager);
	
			var e = passwordManager.enumerator;
	
			var usernames = new Array();
	
			while (e.hasMoreElements()) {
				try {
					var pass = e.getNext().QueryInterface(Components.interfaces.nsIPassword);
					if (pass.host.match(/.* \(ZARAFA_FX\)$/)){
						usernames.push({host: pass.host, user: pass.user});
					}
				} catch (ex) {
				}
			}
			
			for(var i=0;i<usernames.length;i++){
				try {
					passwordManager.removeUser(usernames[i].host, usernames[i].user);
				}catch (ex) {
					alert("Zarafa Add-on Error:\nPREFS 636C 6561 6E50 6173 7377 6F72 644D 616E 6167 6572"+e);
				}
			}
		}else{
			try {
				var loginManager = Components.classes["@mozilla.org/login-manager;1"]
										.getService(Components.interfaces.nsILoginManager);
				var logins = loginManager.getAllLogins({});
				for (var i = 0; i < logins.length; i++) {
					if (logins[i].hostname == "chrome://zarafa"){
						loginManager.removeLogin(logins[i]);
					}
				}
			}catch (ex) {
				alert("Zarafa Add-on Error:\nPREFS 636C 6561 6E50 6173 7377 6F72 644D 616E 6167 6572"+e);
			}
		}
	},
}

// init this stuff 
ZarafaPrefs.init();
