/*
Zarafa Mail Checker
Copyright (C) 2007 Michael Erkens

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/


window.addEventListener("load", ZarafaHierarchy, false);
window.addEventListener("unload", ZarafaHierarchyUnload, false);

function ZarafaHierarchy()
{
	ZarafaSettings.init();
	ZarafaSettings.callWhenReady(ZarafaHierarchyLoadTree);
}

function ZarafaHierarchyUnload()
{
	ZarafaHierarchySaveToFile();
}

function ZarafaHierarchyLoadTree(){
	var xml = '<zarafa><module name="hierarchymodule" id="firefox_2"><action type="list"></action></module></zarafa>';
	Zarafa.request = new XMLHttpRequest();
	Zarafa.request.open("POST", ZarafaPrefs.get("server_url")+"/zarafa.php?subsystem=firefox&external="+ZarafaPrefs.get("server_cookie"), true);
	Zarafa.request.setRequestHeader("Content-Type", "text/xml;charset=utf-8;");
	Zarafa.request.onload = ZarafaHierarchyResponse;
	Zarafa.request.send(xml);
}

function ZarafaHierarchyResponse(e){
	var action = Zarafa.request.responseXML.getElementsByTagName("action")[0];

	var entryids = new Array();

	var storesXML = action.getElementsByTagName("store");
	var stores = new Array();
	for(var s=0;s<storesXML.length;s++){
		var storeID = stores.length;
		stores[storeID] = new Object();

		var store = storesXML[s];
		var storeData = new Object();

		for(var i=0;i<store.attributes.length;i++){
			var attribute = store.attributes[i];
			storeData[attribute.nodeName] = attribute.nodeValue;
		}

		storeData["defaultfolders"] = new Object();
		var defaultfolders = store.getElementsByTagName("defaultfolders")[0];
		for(var i = 0; i < defaultfolders.childNodes.length; i++) {
			var folder = defaultfolders.childNodes[i];
			if(folder.firstChild) {
				storeData["defaultfolders"][folder.firstChild.nodeValue] = folder.nodeName;
			}
		}

		stores[storeID]["data"] = storeData;
		stores[storeID]["folders"] = new Array();
		
		var folders = store.getElementsByTagName("folder");
		for(var i=0; i<folders.length; i++){
			var entryid = Zarafa.getXMLValue(folders[i], "entryid", null);
			if (entryid){
				entryids.push(entryid);

				var parentEntryid = Zarafa.getXMLValue(folders[i], "parent_entryid", null);
				var data = new Object();

				data["parent_entryid"] = parentEntryid;
				data["entryid"] = entryid;

				data["name"] = Zarafa.getXMLValue(folders[i], "display_name", "No Name");
				if (data["name"]=="IPM_SUBTREE"){
					data["name"] = storeData["name"];
				}

				data["class"] = Zarafa.getXMLValue(folders[i], "container_class", "IPF");
				data["count"] = parseInt(Zarafa.getXMLValue(folders[i], "content_count", "0"),10);
				data["unread"] = parseInt(Zarafa.getXMLValue(folders[i], "content_unread", "0"),10);
				data["subfolders"] = (Zarafa.getXMLValue(folders[i], "subfolders", "false")=="true");
				data["open"] = ZarafaSettings.get("folders/entryid_"+entryid+"/is_open") == "1";
				
				stores[storeID]["folders"].push(data);
			}
		}	
	}

	stores.sort(ZarafaSortStores);


	// display store info
	for(var i=0;i<stores.length;i++){
		var store = stores[i];
		
		for(var j=0;j<store["folders"].length;j++){
			var folder = store["folders"][j];
			
			ZarafaAddFolder(folder, store);
		}
	}

	window.ZarafaHierarchyEntryid = entryids;
	ZarafaHierachyLoadFromFile();
}

function ZarafaAddFolder(folder, store)
{
	if (!document.getElementById(folder["entryid"]) && !document.getElementById(folder["parent_entryid"])){
		var parentFolder = ZarafaFindParent(folder["parent_entryid"], store["folders"]);
		if (parentFolder == false){
			ZarafaAddChild(folder["parent_entryid"], folder["entryid"], folder, store["data"]);
		} else{
			ZarafaAddFolder(parentFolder, store);
		}
	}
	if (!document.getElementById(folder["entryid"])){
		ZarafaAddChild(folder["parent_entryid"], folder["entryid"], folder, store["data"]);
	}
}

function ZarafaFindParent(searchId, folders)
{
	for(var i=0;i<folders.length;i++){
		if (folders[i]["entryid"] == searchId)
			return folders[i];
	}
	return false;
}


function ZarafaSortStores(a, b)
{
	storeA = a["data"];
	storeB = b["data"];

	// sort default store always as first
	if(storeA["type"]=="default") return -1;
	if(storeB["type"]=="default") return 1;

	// sort public store always as last
	if(storeA["type"]=="public") return 1;
	if(storeB["type"]=="public") return -1;

	// sort other folders after other stores
	if(storeA["foldertype"]!="all" && storeB["foldertype"]=="all") return 1;
	if(storeA["foldertype"]=="all" && storeB["foldertype"]!="all") return -1;

	// else sort on name
	if(storeA["name"] > storeB["name"]) return 1;
	if(storeA["name"] < storeB["name"]) return -1;
	return 0;
}

function ZarafaAddChild(parentId, newId, data, storeData)
{
	var parentElement = document.getElementById("childs_"+parentId);

	var isStore = storeData["subtree"]==newId;
	if (!parentElement){
		var pEl = document.getElementById(parentId);
		if (pEl){
			pEl.setAttribute("container", "true");
			if (pEl.data["open"]){
				pEl.setAttribute("open", "true");
			}

			parentElement = document.createElement("treechildren");
			parentElement.setAttribute("id", "childs_"+parentId);
			pEl.appendChild(parentElement);
		}else{
			parentElement = document.getElementById("folderroot");
		}
	}

	var isStore = storeData["subtree"]==newId;

	switch(storeData["defaultfolders"][newId]){
		case "inbox":
			if (storeData["type"]=="default"){
				data["active"] = true;
			}
		case "outbox":
		case "sent":
		case "wastebasket":
		case "calendar":
		case "contact":
		case "drafts":
		case "journal":
		case "note":
		case "task":
		case "junk":
			iconFile = "folder."+storeData["defaultfolders"][newId]+".gif";
			break;
		case "publicfolders":
		case "favorites":
			iconFile = "ipf.note.gif";
			break;
		default:
			if (isStore){
				iconFile = "folder.store.gif";
			}else{
				switch(data["class"]){
					case "IPF.Note":
					case "IPF.Contact":
					case "IPF.StickyNote":
					case "IPF.Appointment":
					case "IPF.Task":
					case "IPF.Journal":
						iconFile = data["class"].toLowerCase()+".gif";
						break;
					default:
						iconFile = "ipf.gif";
				}
			}
	}

	var iconURI = "chrome://zarafa/skin/icons/";

	var folder = document.createElement("treeitem");
	folder.setAttribute("id", newId);
	folder.data = data;
	var treerow = document.createElement("treerow");
	if (data["unread"]>0){
		treerow.setAttribute("properties", "unread");
	}

	// folder name
	if (isStore && storeData["type"]=="other"){
		data["name"] +=" of "+storeData["username"];
	}
	var treecell_name = document.createElement("treecell");
	treecell_name.setAttribute("class","treecol-image "+data["class"].replace("."," ").toLowerCase());
	treecell_name.setAttribute("label", data["name"]);
	if (data["unread"]>0){
		treecell_name.setAttribute("properties", "unread");
	}

	treecell_name.setAttribute("src", iconURI+iconFile);
	treerow.appendChild(treecell_name);

	// unread count
	var treecell_unread = document.createElement("treecell");
	treecell_unread.setAttribute("label", data["unread"]);
	if (data["unread"]>0){
		treecell_unread.setAttribute("properties", "unread");
	}
	treerow.appendChild(treecell_unread);

	// total count
	var treecell_count = document.createElement("treecell");
	treecell_count.setAttribute("label", data["count"]);
	if (data["unread"]>0){
		treecell_count.setAttribute("properties", "unread");
	}
	treerow.appendChild(treecell_count);

	// checkbox
	var treecell_active = document.createElement("treecell");
	treecell_active.setAttribute("value", data["active"]?"true":"false");
	treecell_active.setAttribute("id", newId+"_active");
	if (data["unread"]>0){
		treecell_active.setAttribute("properties", "unread");
	}
	treerow.appendChild(treecell_active);

	folder.appendChild(treerow);
	parentElement.appendChild(folder);
}

function ZarafaHierarchySaveToFile()
{
	var entryids = window.ZarafaHierarchyEntryid;
	var data = new Array();
	for(var i=0;i<entryids.length;i++){
		var element = document.getElementById(entryids[i]+"_active");
		if (element&&element.getAttribute("value")=="true"){
			data.push(entryids[i]);
		}
	}
	data = data.join(",");

	var prefs = Components.classes["@mozilla.org/preferences-service;1"]
        .getService(Components.interfaces.nsIPrefService);

	var file;

	// retrieve filename/pointer
	if (prefs.prefHasUserValue("zarafa.hierarchy.filename")){
		file = prefs.getComplexValue("zarafa.hierarchy.filename", Components.interfaces.nsILocalFile);	
	}else{
		file = Components.classes["@mozilla.org/file/directory_service;1"]
			.getService(Components.interfaces.nsIProperties)
			.get("ProfD", Components.interfaces.nsIFile);
		file.append("zarafa.hierarchy");
		file.createUnique(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 0664);
	}

	// open file for output
	try{
		var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
	        .createInstance(Components.interfaces.nsIFileOutputStream);
	}catch(ex){
//		alert("Error writing file:\n"+file.path);
		return;
	}
	foStream.init(file, 0x02 | 0x08 | 0x20, 0664, 0);
	
	foStream.write(data, data.length);
	foStream.close();

	prefs.setComplexValue("zarafa.hierarchy.filename", Components.interfaces.nsILocalFile, file);
}

function ZarafaHierachyLoadFromFile()
{
	var prefs = Components.classes["@mozilla.org/preferences-service;1"]
        .getService(Components.interfaces.nsIPrefService);

	var file = null;

	// retrieve filename/pointer
	if (prefs.prefHasUserValue("zarafa.hierarchy.filename")){
		file = prefs.getComplexValue("zarafa.hierarchy.filename", Components.interfaces.nsILocalFile);	
	}

	if (file == null){
		return;
	}

	var data = "";
	try{
		var fstream = Components.classes["@mozilla.org/network/file-input-stream;1"]
	        .createInstance(Components.interfaces.nsIFileInputStream);
		var sstream = Components.classes["@mozilla.org/scriptableinputstream;1"]
	        .createInstance(Components.interfaces.nsIScriptableInputStream);

		fstream.init(file, -1, 0, 0); // seek to begin
		sstream.init(fstream); 

		var str = sstream.read(4096);
		while (str.length > 0) {
	  		data += str;
			str = sstream.read(4096);
		}

		sstream.close();
		fstream.close();
	}catch(ex){
//		alert("Error reading file:\n"+file.path);
		return;
	}
	
	var eData = data.split(",");
	for(var i=0;i<eData.length;i++){
		var element = document.getElementById(eData[i]+"_active");
		if (element){
			element.setAttribute("value", "true");
		}
	}
}
